#include "StdAfx.h"
#include ".\avisystem.h"

CAviSystem::CAviSystem(void)
{
	memset(&m_Video,0,sizeof(m_Video));
	m_bIsRunning = false;
	
	// Initialising AVIFile library
	AVIFileInit();
}

CAviSystem::~CAviSystem(void)
{
	// Exiting AVIFile library
	AVIFileExit();
}

void CAviSystem::BeginMovieCapture(void)
{
	extern myIDirect3DDevice9* gl_pmyIDirect3DDevice9;
		    
	if (m_bIsRunning) return; 
	
	m_bIsRunning = true;
	
	D3DDISPLAYMODE dMode;
	gl_pmyIDirect3DDevice9->GetDisplayMode(0,&dMode);
		
	m_Video.VideoWidth  = dMode.Width;
	m_Video.VideoHeight = dMode.Height;
	
	m_Video.VideoOK	    = true;
	
	static int i = 0;
	char dummy[255];
	sprintf (dummy, "C:\\myVideo%i.avi",i);

	if(AVIFileOpen(&m_Video.VideoFile, dummy, OF_WRITE | OF_CREATE, NULL) != AVIERR_OK)
	{
		m_Video.VideoOK = false;
		return;
	}

	i++;
}

void CAviSystem::EndMovieCapture(void)
{
	if (!m_bIsRunning) return; 
	
	m_bIsRunning = false;
	
	if(m_Video.Stream)
	{
		AVIStreamClose(m_Video.Stream);
		m_Video.Stream = NULL;
	}

	if(m_Video.CompressionStream)
	{
		AVIStreamClose(m_Video.CompressionStream);
		m_Video.CompressionStream = NULL;
	}

	if(m_Video.VideoFile)
    {
		AVIFileClose(m_Video.VideoFile);
		m_Video.VideoFile = NULL;
    }

	m_Video.VideoOK = false;
	m_Video.NumberOfFrames = 0;
}

void CAviSystem::UpdateMovieCapture(void)
{
	if (!m_Video.VideoOK) return;

	LPBITMAPINFOHEADER ScreenData = (LPBITMAPINFOHEADER)GlobalLock(CaptureScreen());

	if(m_Video.NumberOfFrames == 0)
	{
		memset(&m_Video.StreamInformation, 0, sizeof(AVISTREAMINFO));
		m_Video.StreamInformation.rcFrame.top		    = 0;
		m_Video.StreamInformation.rcFrame.left		    = 0;
		m_Video.StreamInformation.fccHandler			= 0;
		m_Video.StreamInformation.dwScale               = 1;
		m_Video.StreamInformation.dwRate                = 5; // FRAMES PER SECOND 
		m_Video.StreamInformation.rcFrame.right		    = m_Video.VideoWidth;
		m_Video.StreamInformation.rcFrame.bottom	    = m_Video.VideoHeight;
		m_Video.StreamInformation.fccType               = mmioFOURCC('v', 'i', 'd', 's');
		m_Video.StreamInformation.dwSuggestedBufferSize = m_Video.VideoWidth * m_Video.VideoHeight;

		if(AVIFileCreateStream(m_Video.VideoFile, &m_Video.Stream, &m_Video.StreamInformation) != AVIERR_OK)
		{
			GlobalFreePtr(ScreenData);
			m_Video.VideoOK = false;
			return;
		}

		AVICOMPRESSOPTIONS CompressionOptions;
		AVICOMPRESSOPTIONS FAR* CompressionOptionsF[1] = {&CompressionOptions};
		memset(&CompressionOptions, 0, sizeof(CompressionOptions));

		if(AVIMakeCompressedStream(&m_Video.CompressionStream, m_Video.Stream, &CompressionOptions, NULL) != AVIERR_OK)
		{
			GlobalFreePtr(ScreenData);
			m_Video.VideoOK = false;
			return;
		}

		if(AVIStreamSetFormat(m_Video.CompressionStream, 0, ScreenData, ScreenData->biSize + ScreenData->biClrUsed * sizeof(RGBQUAD)) != AVIERR_OK)
		{
			GlobalFreePtr(ScreenData);
			m_Video.VideoOK = false;
			return;
		}
	}

	if(AVIStreamWrite(m_Video.CompressionStream, m_Video.NumberOfFrames++, 1, (unsigned char*)ScreenData + ScreenData->biSize + ScreenData->biClrUsed * sizeof(RGBQUAD), ScreenData->biSizeImage, AVIIF_KEYFRAME, NULL, NULL) != AVIERR_OK)
	{
		GlobalFreePtr(ScreenData);
		m_Video.VideoOK = false;
		return;
	}

	GlobalFreePtr(ScreenData);

}

HANDLE CAviSystem::CaptureScreen(void)
{
	extern myIDirect3DDevice9* gl_pmyIDirect3DDevice9;
		
	HDC SourceHDC = ::GetDC(NULL);

    HDC CompatibleHDC = CreateCompatibleDC(SourceHDC);

	HBITMAP BitmapHandle = CreateCompatibleBitmap(SourceHDC, m_Video.VideoWidth, m_Video.VideoHeight); 
   	if(BitmapHandle == 0 || !SelectObject(CompatibleHDC, BitmapHandle))
    {
		return(NULL);
    }

    if(!BitBlt(CompatibleHDC, 0, 0, m_Video.VideoWidth, m_Video.VideoHeight, SourceHDC, 0, 0, SRCCOPY)) 
    {
		return(NULL);
    }

	BITMAP Bitmap;
    GetObject(BitmapHandle, sizeof(BITMAP), &Bitmap);

	unsigned long BitmapSize = sizeof(BITMAPINFOHEADER) + (((Bitmap.bmWidth * 32 + 31) / 32 * 4) * Bitmap.bmHeight);

	HANDLE MemoryHandle = GlobalAlloc(GHND, BitmapSize);

	if(!MemoryHandle)
	{
		return NULL;
	}

	LPBITMAPINFOHEADER BMInfo = (LPBITMAPINFOHEADER)GlobalLock(MemoryHandle);
	BMInfo->biSizeImage		  = BitmapSize - sizeof(BITMAPINFOHEADER);
	BMInfo->biSize			  = sizeof(BITMAPINFOHEADER);
	BMInfo->biHeight		  = Bitmap.bmHeight;
	BMInfo->biWidth			  = Bitmap.bmWidth;
	BMInfo->biCompression	  = BI_RGB;
	BMInfo->biBitCount		  = 32;
	BMInfo->biPlanes		  = 1;
	BMInfo->biXPelsPerMeter	  = 0;
	BMInfo->biYPelsPerMeter	  = 0;
	BMInfo->biClrUsed	  	  = 0;
	BMInfo->biClrImportant	  = 0;

	GetDIBits(CompatibleHDC, BitmapHandle, 0, Bitmap.bmHeight, (unsigned char*)(BMInfo + 1), (LPBITMAPINFO)BMInfo, DIB_RGB_COLORS);

	DeleteDC(CompatibleHDC);
    DeleteObject(BitmapHandle);
	GlobalUnlock(MemoryHandle);

	return (MemoryHandle);	
}
